/*
                               ConvPL by kravietZ

          Uniwersalny konwerter standardow polskich znakow, freeware.

          Kontakt: Pawel Krawczyk <kravietz@pipeta.chemia.pk.edu.pl>
                                  <2:486/23@fidonet>

          Najnowsza wersja:
          ftp://pipeta.chemia.pk.edu.pl/pub/convpl

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

#ifdef __MSDOS__
#include <io.h>
#include <sys\stat.h>
#include <dos.h>
#undef __GNUC__     /* DJGPP definiuje obydwie zmienne */
#endif

#ifdef __unix__
#include <sys/file.h>
#include <sys/stat.h>
#include <glob.h>
/* nie wlaczac unistd.h, bo bedzie konflikt getopt() */
#endif

#include "polish.h"
#include "disph.h"
#include "opts.h"
#include "version.h"

#define TEMP_FILE "tempxx.cpl"
#define TO_CRLF 1
#define TO_LF 2
#define buf_size 20*1024 /* bufor 20Kb */

typedef unsigned char ubyte;

static char ibuf[buf_size], obuf[buf_size];
static int ibuf_ptr = 0, obuf_ptr = 0, ibuf_lim = 0;
static int f_in=0,f_out=1;
#ifndef CGI
static int opt_mode=0;
#else
static int opt_mode=TO_LF;
#endif

#ifdef __unix__
glob_t globbuf;
struct stat statbuf;
#endif

#ifdef __MSDOS__ /* to do przyszlych zastosowan... */
const char f_ext_check[14],bin_ext_lst[]=".exe .com .gif .jpg";
#endif

int indexx(ubyte ch, ubyte *ctab) /* czesto uzywana funkcja => inline */
/*
    zwraca indeks znaku ch w stringu okreslonym przez wskaznik ctab
    lub -1 jesli nie znajdzie znaku; indeksy zaczynaja sie od 0;
*/
{ int poss=0;
if(ctab == NULL) return(-1);
while((ch != *(ctab+poss) && (*(ctab+poss) != '\0'))) poss++;
if(*(ctab+poss) == '\0') return(-1);
 else
return(poss);
}; /* postab */

ubyte getxx (void)
{ int i;
  if (ibuf_ptr >= ibuf_lim) {               /* odebrane wszystkie? */
    ibuf_lim = read(f_in, ibuf, buf_size);  /* czytaj z pliku */
    if (ibuf_lim == 0) return 0;            /* nie ma wiecej w pliku */
    ibuf_ptr = 0;                           /* inicjuj wskaznik znakow */
  }
  return ibuf[ibuf_ptr++];                  /* daj znak, zwieksz wskaznik */
}

void putxx (ubyte ch)
{
  if (obuf_ptr >= buf_size) {                /* bufor pelny? */
    write(f_out, obuf, obuf_ptr);           /* zapisz bufor do pliku */
    obuf_ptr = 0;                           /* inicjuj wskaznik znakow */
  }
  obuf[obuf_ptr++] = ch;                    /* znak do bufora, zwieksz wsk. */
}

int ConvFile(char *cf_fname,char *cf_fnameo,char st1,char st2)
/*  
    czyta z nazwy: cf_fname
    zapisuje do nazwy: cf_fnameo
    st1 = std zrodlowy,
    st2 = docelowy
    parametr z zewnatrz (globalny): opt_mode (konwersja koncow linii)
*/
{
    ubyte *in_tab, *out_tab;
    int ps1;
    char ch1;
    long flen;

#ifndef NO_LINE_CNT
    long int n_line=0;  /* numer linii */
#endif

ibuf_ptr = obuf_ptr = ibuf_lim = 0;

switch(st1)     /* najpierw kontrola poprawnosci standardow */
    {

/*  drobny trick umozliwiajacy przekodowanie u*x <-> MSDOS bez
    zmiany standardu polskich liter; szczegoly w cpl.txt */

    case '0': in_tab=p_Asc; st2='0'; break;
    case '1': in_tab=p_Maz; break;
    case '2': in_tab=p_FMaz; break;
    case '3': in_tab=p_IBML; break;
    case '4': in_tab=p_ISOL; break;
    case '5': in_tab=p_DHN; break;
    case '6': in_tab=p_CSK; break;
    case '7': in_tab=p_Cyf; break;
    case '8': in_tab=p_IEA; break;
    case '9': in_tab=p_Log; break;
    case 'a': in_tab=p_Mic; break;
    case 'b': in_tab=p_Ami; break;
    case 'c': in_tab=p_CPJ; break;
    case 'd': in_tab=p_FAT; break;
    case 'e': in_tab=p_Win3; break;
    case 'f': in_tab=p_XJP; break;
    case 'g': in_tab=p_Ven; break;
    case 'h': in_tab=p_Mac; break;
    case 'i': in_tab=p_TeX; break;
    case 'j': in_tab=p_Cor; break;
    case 'k': in_tab=p_Cal; break;
    case 'l': in_tab=p_FX; break;
    case 'm': in_tab=p_KWK; break;
    case 'n': in_tab=p_MSW; break;
    case 'o': in_tab=p_ATM; break;
    default: fprintf(stderr,"\007\b\b\b\b\bnieznany standard wejsciowy \"%c\".\n",st2); return(255);
    } /* switch st1 */

switch(st2)
    {
    case '0': out_tab=p_Asc; break;
    case '1': out_tab=p_Maz; break;
    case '2': out_tab=p_FMaz; break;
    case '3': out_tab=p_IBML; break;
    case '4': out_tab=p_ISOL; break;
    case '5': out_tab=p_DHN; break;
    case '6': out_tab=p_CSK; break;
    case '7': out_tab=p_Cyf; break;
    case '8': out_tab=p_IEA; break;
    case '9': out_tab=p_Log; break;
    case 'a': out_tab=p_Mic; break;
    case 'b': out_tab=p_Ami; break;
    case 'c': out_tab=p_CPJ; break;
    case 'd': out_tab=p_FAT; break;
    case 'e': out_tab=p_Win3; break;
    case 'f': out_tab=p_XJP; break;
    case 'g': out_tab=p_Ven; break;
    case 'h': out_tab=p_Mac; break;
    case 'i': out_tab=p_TeX; break;
    case 'j': out_tab=p_Cor; break;
    case 'k': out_tab=p_Cal; break;
    case 'l': out_tab=p_FX; break;
    case 'm': out_tab=p_KWK; break;
    case 'n': out_tab=p_MSW; break;
    case 'o': out_tab=p_ATM; break;
    default: fprintf(stderr,"\007\b\b\b\b\bnieznany standard wyjsciowy \"%c\".\n",st2); return(255);
    } /* switch st2 */

    

if(!strcmp(cf_fname,"stdin")) 

    f_in=0;

    else 
    { 
    f_in=open(cf_fname,O_RDONLY

    #ifdef __MSDOS__
           |O_BINARY
    #endif

    );
    fprintf(stderr,"%s:      ",cf_fname);
#ifdef __unix__
    stat(cf_fname,&statbuf);
#endif

    }
    
if(f_in != -1) 

    {

if(!strcmp(cf_fnameo,"stdout"))

    f_out=1;

    else

    f_out=open(cf_fnameo,O_RDWR|O_CREAT

#ifdef __MSDOS__
       |O_BINARY
#endif

#ifdef __unix__
      ,statbuf.st_mode
#endif

    );

    if(S_ISDIR(statbuf.st_mode))
    { fprintf(stderr, "pomijam\n");
      return;
    }

    if(f_out != -1)
	{

while((ch1=getxx()) != 0)

    {   /* opt_mode = 0 jesli nie crlf, 1,2 jesli konw. crlf */

    switch(opt_mode) {

case TO_CRLF: { if(ch1==10) { putxx(13); putxx(10);

#ifndef NO_LINE_CNT
    n_line++; fprintf(stderr,"\b\b\b\b\b%-5u",n_line);
#endif

	 } else { if( (ps1=indexx(ch1,in_tab)) != -1 ) putxx(*(out_tab+ps1)); else putxx(ch1); }; break; }

case TO_LF: { if(ch1==13) {

#ifndef NO_LINE_CNT
        n_line++; fprintf(stderr,"\b\b\b\b\b%-5u",n_line);
#endif

	   break; } else { if( (ps1=indexx(ch1,in_tab)) != -1 ) putxx(*(out_tab+ps1)); else putxx(ch1); }; break; }

default: { if( (ps1=indexx(ch1,in_tab)) != -1 ) putxx(*(out_tab+ps1)); else putxx(ch1);

#ifndef NO_LINE_CNT
        n_line++; fprintf(stderr,"\b\b\b\b\b%-5u",n_line);
#endif

	  break; }

    } /* switch */

}; /* while */

/* zrzucamy bufor do pliku */

    if (obuf_ptr < buf_size) {

    write(f_out, obuf, obuf_ptr);           /* zapisz bufor do pliku */
        }

       close(f_out);

     } /* open(f_out) */

     else { perror(cf_fnameo); /* exit(1); */ }

       close(f_in);

       /* jesli nie stdin napisz ok */
       if(f_in) fprintf(stderr,"ok.\n");

     } /* open(f_in) */

else { perror(""); /* exit(1); */ }

return(0);

} /* convfile */

int main(argc, argv)
int argc;
char **argv;
{

char swt[4];
char st_in,st_out;
int i,rargc1;

    char valid1[]="0123456789abcdefghijklmnoABCDFSL";
    char valid2[]="0123456789abcdefghijklmno-=";

if(argc>1) /* nie rargc! tu licza sie takze opcje */
{

/* sprawdz rozne mniej lub bardziej standardowe opcje */
if(gopt2("-?") || gopt2("--help") || gopt2("-h") )
                                                { 
                                                DispLogo();
                                                DispH(); 
                                                exit(1); }

if(gopt2("-V") || gopt2("--version") || gopt2("--wersja") )
    { fprintf(stdout,CPL_VER); exit(0); }

if(gopt2("-S") || gopt2("--standard"))
     { DispS(); exit(0); }

if(gopt2("-L") || gopt2("--licencja"))
    { DispL(); exit(0); }

rargc1 = rargc;
if(rargc1<2) { 
             fprintf(stderr,"Musisz podac przynajmniej rodzaj konwersji.\n"
             "Polecenie \"cpl -h\" wyswietli wszystkie dostepne opcje.\n");
               exit(1);
             }

/* wywolanie "cpl nn -", czeste pod unixem */
if(gopt2("-")) if(argc == 3) argc--; 
  else 
    { fprintf(stderr,"Musisz podac konwersje. cpl -h wyswietli wszystkie dostepne konwersje.\n");
    exit(1); }

/* sprawdz konwersje koncow linii */
if(indexx('=',getarg(1,argv,argc)) != -1)
                                opt_mode=TO_CRLF;

if(indexx('-',getarg(1,argv,argc)) != -1)
                                opt_mode=TO_LF;

/* sprawdzanie makr */

switch(*argv[1])

{
    case 'A':   strncpy(swt,"13",2); break;
    case 'B':   strncpy(swt,"31",2); break;
    case 'C':   strncpy(swt,"10",2); break;
    case 'D':   strncpy(swt,"30",2); break;
    case 'F':   strncpy(swt,"21",2); break;
    default:    strcpy(swt,garg(1));

}; /* sprawdzanie makr */

st_in=swt[0];
st_out=swt[1];

/*  wszystkie ew. przypisania zostaly dokonane; teraz sprawdzamy
    poprawnosc argumentow; jesli CGI to bledy pojda do logow */

if( (indexx(st_in,valid1) == -1) || (indexx(st_out,valid2) == -1))
 { fprintf(stderr,"Zly parametr \"%c%c\". Polecenie: cpl -?  wyswietli\
 liste dostepnych opcji.\n",st_in,st_out); exit(1); }



if(rargc1==2) {
             ConvFile("stdin","stdout",st_in,st_out );
             unlink("stdout");
             exit(0);
             }

  else

   if(rargc1>=3) {
                 DispLogo();

                /* rozwin pierwszy argument po nn (nn=0) */

                if(glob(getarg(2,argv,argc), 0, NULL, &globbuf) != 0) 
                perror("glob1");

                if (rargc1>3) 
                { 
                        
                    for(i=3; i<rargc1; i++) 
                    {
                    if( glob(getarg(i,argv,argc), GLOB_APPEND, NULL, &globbuf)
                    != 0) perror("globi");
                    }
                }

                for(i=0; i<globbuf.gl_pathc; i++) 

                     {
                     ConvFile(globbuf.gl_pathv[i], TEMP_FILE, st_in, st_out );
                     unlink(globbuf.gl_pathv[i]);
                     rename(TEMP_FILE, globbuf.gl_pathv[i]);
                     }

                }

} /* argc>1 */

      else

{
    DispLogo();
    DispH();
    return(1);
} /* argc<2 */

globfree(&globbuf);

return(0);

} /* main */
